@@ -9,12 +9,16 @@ AUTHOR_STRINGS = (
     'subtitles by',
     'encoded and released by',
     'opensubtitles.org',
-    'please rate this subtitle'
+    'please rate this subtitle',
+    'captioning sponsored by',
+    'captioned by',
 )
 
+
 class Subtitle:
     '''Subtitle contents object
-        (invidual subtitle entry)'''
+    (invidual subtitle entry)
+    '''
 
     def __init__(self):
         self._index = None
@@ -23,9 +27,7 @@ class Subtitle:
         self.end = None
 
     def __str__(self):
-        return '{}\n{} --> {}\n{}\n'.format(
-            self._index, self.start, self.end, self._contents
-        )
+        return '{}\n{} --> {}\n{}\n'.format(self._index, self.start, self.end, self._contents)
 
     def __eq__(self, other):
         if self.__str__() == other.__str__():
@@ -42,7 +44,7 @@ class Subtitle:
 
     @property
     def index(self):
-        '''Returns the index number for subtitle, or False is index is not assigned'''
+        '''Returns the index number for subtitle, or False if index is not assigned'''
         if self._index is None:
             return False
         return self._index
@@ -65,7 +67,8 @@ class Subtitle:
 
     def _filter_empty(self):
         '''Removes empty quotes from contents list,
-            Converts self.index to 0'''
+        Converts self.index to 0
+        '''
         # Set index as 0 for later deletion
         if not self.contents:
             self.index = 0
@@ -79,11 +82,24 @@ class Subtitle:
             *self._contents.split('\n'),
         ]
 
+    @staticmethod
+    def _remove_comma_space(matchobj):
+        return matchobj.group(0).replace(' ,', ',')
+
+    @staticmethod
+    def _add_comma_space(matchobj):
+        return matchobj.group(0).replace(',', ', ')
+
+    def fix_comma_spaces(self):
+        '''Fixes comma space seperation'''
+        for _ in re.findall(r'[A-Za-z]+\s+,', self._contents):
+            self._contents = re.sub(r'[A-Za-z]+\s+,', self._remove_comma_space, self._contents)
+        for _ in re.findall(r'[A-Za-z]+,[A-Za-z]+', self._contents):
+            self._contents = re.sub(r'[A-Za-z]+,[A-Za-z]+', self._add_comma_space, self._contents)
+
     def remove_font_colours(self):
         '''Removes <font> tags from contents'''
-        self._contents = re.sub(
-            r'\<font(.*)\>(.*)\</font\>', '', self._contents, flags=re.DOTALL
-        )
+        self._contents = re.sub(r'\<font(.*)\>(.*)\</font\>', '', self._contents, flags=re.DOTALL)
         self._filter_empty()
 
     def remove_asterisks(self):
@@ -93,37 +109,49 @@ class Subtitle:
 
     def remove_music(self):
         '''Removes music symbols from contents'''
+        # Remove music symbol behaving as parenthesis
+        self._contents = re.sub(r'♪(.*)♪', '', self._contents, flags=re.DOTALL)
+        # Remove behaving as inline
         self._contents_to_list()
         for idx, _ in enumerate(self._contents):
-            if '♪' in self._contents[idx]:
+            if any(symbol in self._contents[idx] for symbol in ['#', '♪']):
                 self._contents[idx] = ''
         self._contents_to_str()
         self._filter_empty()
 
     def remove_sound_effects(self):
-        '''Removes text in between parenthesis and square brackets'''
+        '''Removes text in between parenthesis, brackets, and forward slashes'''
         # Remove single line brackets
         self._contents_to_list()
         for idx, _ in enumerate(self._contents):
-            self._contents[idx] = re.sub(
-                r'[\(\[][\S ]*[\)\]][\s]*', '', self._contents[idx]
-            )
+            # Have split this check into a for loop across the delimiters as providing
+            # them in one regex expression will yield errors for forward slash within italics
+            # tag when used with square brackets/parenthesis. e.g line 8 of
+            # subtitle_sound_effects_before.srt
+            for prefix, suffix in (('(', ')'), ('[', ']'), ('/', '/')):
+                self._contents[idx] = re.sub(
+                    rf'[\{prefix}][\S ]*[\{suffix}][\s:]*', '', self._contents[idx]
+                )
         self._remove_lone_symbols()
         self._contents_to_str()
         # Remove multi-line brackets
-        self._contents = re.sub(r'[\(\[][\S\s]*[\)\]][\s]*', '', self._contents)
+        self._contents = re.sub(r'[\(\[][\S\s]*[\)\]][\s:]*', '', self._contents)
         self._filter_empty()
 
     def replace_names(self):
-        '''Replace names in all caps with dashes'''
-        names = []
-        names.extend(re.findall(r'([A-Z ]+ *: *|[A-Z]{1}[a-z]+: *)', self._contents))
+        '''Replace names in all caps'''
+        # Care is taken here to preserve genuine sentences with a colon.
+        names = re.findall(r'([A-Z0-9 ]+ *: *|[A-Z]{1}[a-z]+ *: *)', self._contents)
         if len(names) > 1:
             # Replace names with '- '
-            self._contents = re.sub(r'([A-Z ]+ *: *|[A-Z]{1}[a-z]+: *)', '- ', self._contents).lstrip()
+            self._contents = re.sub(
+                r'([A-Z0-9 ]+ *: *|[A-Z]{1}[a-z]+ *: *)', '- ', self._contents
+            ).lstrip()
         else:
             # Replace name with empty string.
-            self._contents = re.sub(r'([A-Z ]+ *:|[A-Z]{1}[a-z]+: *)', '', self._contents).lstrip()
+            self._contents = re.sub(
+                r'([A-Z0-9 ]+ *: *|[A-Z]{1}[a-z]+ *: *)', '', self._contents
+            ).lstrip()
         self._filter_empty()
 
     def remove_author(self):
@@ -133,15 +161,25 @@ class Subtitle:
                 self.index = 0
                 break
 
-    def remove_italics(self):
-        '''Removes empty <i> tags, and empty dashes'''
-        self._contents = re.sub(r'<i>[\s]*</i>', '', self._contents, flags=re.DOTALL)
+    def fix_italics(self):
+        '''Fixes lone <i> or </i> tags, and removes empty <i> tags, and empty dashes'''
+        if '<i>' in self._contents and '</i>' not in self._contents:
+            self._contents += '</i>'
+        if '</i>' in self._contents and '<i>' not in self._contents:
+            self._contents = '<i>' + self._contents
+        self._contents = re.sub(r'<i>[\_\-\‐\?#\s¶]*</i>', '', self._contents, flags=re.DOTALL)
         self._remove_lone_symbols()
 
     def _remove_lone_symbols(self):
         self._contents_to_list()
         for idx, _ in enumerate(self._contents):
-            self._contents[idx] = re.sub(r'^[\_\-\?#\s¶]*$', '', self._contents[idx])
+            self._contents[idx] = re.sub(r'^[\_\-\‐\?#\s¶]*$', '', self._contents[idx])
+            self._contents[idx] = re.sub(
+                r'^[\_\-\‐\?#\s¶]*<i>[\_\-\‐\?#\s¶]*$', '<i>', self._contents[idx]
+            )
+            self._contents[idx] = re.sub(
+                r'^[\_\-\‐\?#\s¶]*</i>[\_\-\‐\?#\s¶]*$', '</i>', self._contents[idx]
+            )
         # Removes empty strings
         self._contents = list(filter(None, self._contents))
         # Set index as 0 for later deletion
@@ -162,9 +200,7 @@ class Subtitles:
             raise IOError('{} is not a file'.format(fpath))
         self._fullpath = fpath
         if self.ext not in self.EXTENSIONS:
-            raise IOError(
-                '{} is not valid subtitle file: {}'.format(self._fullpath, self.ext)
-            )
+            raise IOError('{} is not valid subtitle file: {}'.format(self._fullpath, self.ext))
         self._line_list = self._get_line_list()
         self.subtitles = self._parse_subs()
 
@@ -188,7 +224,7 @@ class Subtitles:
         return ext
 
     def _get_line_list(self):
-        with codecs.open(self.filepath, 'r', encoding='utf-8', errors='ignore') as fdata:
+        with codecs.open(self.filepath, 'r', encoding='utf-8-sig', errors='ignore') as fdata:
             line_list = fdata.readlines()
         line_list_filtered = [x.rstrip() for x in line_list]
         return line_list_filtered
@@ -203,8 +239,11 @@ class Subtitles:
                 except ValueError:
                     continue
             # Time line
-            elif ' --> ' in line:
-                sub_list[-1].start, sub_list[-1].end = line.split(' --> ')
+            elif sub_list[-1].start is None:
+                if ' --> ' in line:
+                    sub_list[-1].start, sub_list[-1].end = line.split(' --> ')
+                else:
+                    continue
             # New subtitle entry
             elif not line:
                 sub_list.append(Subtitle())
@@ -228,7 +267,9 @@ class Subtitles:
             any(map(lambda sub: sub.replace_names(), self.subtitles))
         if kw.get('rm_author', True):
             any(map(lambda sub: sub.remove_author(), self.subtitles))
-        any(map(lambda sub: sub.remove_italics(), self.subtitles))
+        if kw.get('fix_commas', True):
+            any(map(lambda sub: sub.fix_comma_spaces(), self.subtitles))
+        any(map(lambda sub: sub.fix_italics(), self.subtitles))
         # Remove filtered items from list
         self.subtitles[:] = [sub for sub in self.subtitles if sub.index]
         # Reassign indices
@@ -242,10 +283,10 @@ class Subtitles:
 
     def save(self, new_filepath=None):
         '''Saves subtitle object to disk,
-            omit new_filepath to save inplace
+        omit new_filepath to save inplace
         '''
         if new_filepath is not None:
             self._fullpath = new_filepath
-        with open(self._fullpath, 'w') as fp:
+        with open(self._fullpath, 'w', encoding='utf-8') as fp:
             for sub in self.subtitles:
                 fp.write(str(sub) + '\n')
